<?php
/*
 * Created on Jun 1, 2006
 *
 *
 * Kod_Form.php - kodform4
 * Copyright (C) 2006 boris
 *
 */
/**
 * @package Kod_Form
 * 
 * This file contains what is called Kod_Form library. That is
 * library for creating and validating HTML forms. It has
 * server side validation but not client side. You can always
 * use some javascript spcrips for cilent validation.
 * 
 * @author Boris Tomic <kodmasin@gmail.com>
 * 
 */

/**
 * class which implements form element
 */
class Kod_Form{
	/**
	 * @var array Holds form elements
	 */
	private $elements;
	/**
	 * @var array form errors
	 * 
	 * This is array in form
	 * <code>
	 * 
	 * </code>
	 */
	private $errors;
	/**
	 * @var string of form used for name attribute
	 */
	private $id;
	/**
	 * @var string holds action value. That is action form
	 * tag attribute. 
	 */
	private $action;
	/**
	 * @var integer holds method information
	 * 
	 * it holds index of method defined in $methods member
	 * 
	 * @see $methods
	 */
	private $method;
	/**
	 * @var array submit form element added to this form
	 */
	private $submits;
	/**
	 * @var mixed
	 */
	private $valid;
	/**
	 * @var array available methods for form tag attribute method
	 * @static
	 * 
	 */
	public $methods = array("post", "get");
	/**
	 * Creates new form element
	 * 
	 * @param string $id id (name) of form
	 * @param integer $method defines method (0 => post, 1 => get)
	 * @param string $action this is url where form should post infact
	 * 	this is action attribute
	 */
	public function __construct($id, $method = 0, $action = null){
		$this->elements = array();
		$this->errors = array();
		$this->submits = array();
		$this->id = $id;
		$this->method = $method;
		$this->action = $action;
		$this->valid = null;
	}
	/**
	 * check if this form is valid
	 * 
	 * It check if form has already been validated if yes
	 * it returns that time validation result. If form has
	 * not been validated validate form and reutrn result
	 * 
	 * @return boolean true if valid false iuf not 
	 */
	public function isValid(){
		if($this->valid !== null)
			return $this->valid;
		$this->valid = true;
		foreach($this->elements as &$element){
			if(!$element->validate()){
				$this->valid = false;
				$this->errors[$element->getDisplayName()] = $element->getErrors();
			}
		}
		return $this->valid;
	}
	/**
	 * This function is itended to be used if form is 
	 * extended by another class
	 * 
	 * I Am not shure will this be implemented so for
	 * now this is just place holder.
	 */
	public function process(){
		return ;
	}
	/**
	 * Adds new element to form
	 * 
	 * @param Kod_Form_Element $element this is new element to be
	 * added to this form
	 * @return Kod_Form_element $element returns same element which is added
	 */
	public function &addElement(Kod_Form_Element &$element){
		if($element instanceof Kod_Submit)
			$this->submits[$element->getId()] =& $element;
		else{ 
			$this->elements[$element->getId()] =& $element;
		}
		$element->setForm($this);
		return $element;
	}
	/**
	 * Returns all errors of this form.
	 * 
	 * It should be called after isValid method othervise it
	 * will return empty array which toes not neceseary mean
	 * that values in form are valid
	 * 
	 * @return array in form:
	 * <code>
	 * array("element1_name" => array("Error1", "Error2"),
	 * 		"element2_name" => array("Error3", "Error"));
	 * </code>
	 */
	public function getErrors(){
		return $this->errors;
	}
	/**
	 * sets charachter which will be used for marking
	 * required fields.
	 */
	public function setRequiredMark($string){
		$this->requiredMark = $string;
	}
	/**
	 * checks if this form has required elements
	 * 
	 * @return boolean true if form has required elements
	 * and false if not
	 */
	public function hasRequiredElements(){
		$yes = false;
		foreach($this->elements as &$element)
			if($element->isRequired())
				$yes = true;
		return $yes;
	}
	/**
	 * checks if this form is submited
	 * 
	 * @return boolean returns true if form is submited
	 * and fals eif it is not
	 */
	public function isSubmited(){
		foreach($this->submits as $submit){
			if($submit->isSubmited())
				return true;
		}
		return false;
	}
	/**
	 * returns method which this form is using (post, get)
	 */
	public function getMethod(){
		return $this->methods[$this->method];
	}
	public function getAction(){
		return $this->action;
	}
	/**
	 * returns POST or GET array.
	 * 
	 * Depends on which method is set
	 */
	public function &getRawData(){
		if($this->method == 0)
			return $_POST;
		if($this->method == 1)
			return $_GET;
		return null;
	}
	/**
	 * returns raw (submited) data of form element
	 * 
	 * @param string $id name of form element
	 * @return mixed raw value of form element or null if 
	 * element does not exist in this form 
	 */
	public function getRaw($id){
		$data = $this->getRawData();
		if(array_key_exists($id, $data))
			return $data[$id];
		return null;
	}
	/**
	 * function for quick rendering of form (for fast prototyping) but 
	 * you can provide any template you like
	 * @param string $template path to form template
	 */
	public function display($template = null){
		if($template == null)
			$template = dirname(__FILE__) . DIRECTORY_SEPARATOR . "Kod_Form_Def.php";
		ob_start();
		if(file_exists($template))
			require $template;
		return ob_get_clean();
	}
	public function clear(){
		foreach($this->elements as &$element)
			$element->clear();
	}
	public function &__get($name){
		if(array_key_exists($name, $this->elements))
			return $this->elements[$name];
		throw new Exception("There is no element under name " . $name . " in this form!.");
	}
	/**add error to from errors
	 * this function add error to form errors and invalidate form
	 * Use it when you have to process from data but after that you find that data is not valid
	 * fopr ex. login form where you get invalid after you check login data*/
	public function addError($name, $message){
		$this->errors[$this->id . $name] = array($message);
		$this->valid = false;
	}
}

/**
 * base class for all Kod_Form elements
 */
abstract class Kod_Form_Element{
	/**
	 * @var Kod_Form holds parent form element
	 */
	protected $form;
	/**
	 * @var array holds list of validators
	 */
	protected $validators;
	/**
	 * @var array holds list of filters
	 */
	protected $filters;
	/**
	 * @var string holds submited (raw) data of element.
	 * 
	 * This variable is only set if form is submited and 
	 * validate method is called (ususaly by parent form in 
	 * isValid). 
	 * Otherwise it holds null 
	 */
	protected $rawValue;
	/**
	 * @var mixed holds value of form which is valid.
	 * 
	 * It is set
	 * if form is submited and value valid (in validate method after
	 * successful validatio). Otherwise it holds null
	 */
	protected $validValue;
	/**
	 * @var mixed this holds default value
	 * 
	 * This value can be set in constructor or vith memeber methods.
	 * It should be used when displaying form only if form is no
	 * submited.
	 */
	protected $defaultValue;
	/**
	 * @var array holds error messages generated by validators
	 * This value is available after calling validate method because
	 * validators are coled there
	 */
	protected $errors;
	/**
	 * @var string holds name of element
	 */
	protected $id;
	/**
	 * @var Required_Validator holds required validator. Element
	 * can have only one validator of this type and it is holded here.
	 */
	protected $required;
	/**
	 * @var string This is display name of this element
	 * 
	 * It is used when comunicating with users (generating errors, ...)
	 */
	protected $displayName;
	/**
	 * creates Kod_Form_Element element
	 * 
	 * @param string $id name of this element, also used for name 
	 * attribute in html
	 * @param string $defaultValue holds default value of element
	 * @param string $required sets required validator and assign this
	 * string as not valid message   
	 * 
	 */
	function __construct($id, $defaultValue = null, $required = null){
		$this->validators = array();
		$this->filters = array();
		$this->errors = array();
		$this->value = null;
		$this->defaultValue = $defaultValue;
		$this->rawValue = null;
		$this->id = $id;
		$this->form = null;
		$this->required = null;
		if($required != null )
			$this->setRequired($required);
		$this->validValue = null;
		$this->displayName = null;
	}
	/**
	 * sets the parent form
	 */
	function setForm(&$form){
		$this->form =& $form;
	}
	/**
	 * validate this form element
	 * 
	 * Ususaly called by Kod_Form::isValid
	 * If form is not submite this function alvays returns true.
	 * 
	 * @return boolean if form element valid returns true otherwise false
	 * 
	 */
	function validate(){
		$valid = true;
		//check if form is submited
		if($this->form != null && $this->form->isSubmited()){
			$this->rawValue = $this->form->getRaw($this->id);//get raw submited data
			if($this->required != null){//if required do required validation
				if(!$this->required->validate($this->rawValue)){
					//$valid = false;
					array_push($this->errors, $this->required->getError());
					return false;
				}
			}
			//filter raw data
			$filteredValue = $this->rawValue;
			foreach($this->filters as $filter){
				$filteredValue = $filter->filter($filteredValue);
			}
			//validate with "classic" validators (not required one)
			foreach($this->validators as $validator){
				if(!$validator->validate($filteredValue)){
					$valid = false;
					array_push($this->errors, $validator->getError());
				}
			}
			if($valid)
				$this->validValue = $filteredValue;
		}
		return $valid;
	}
	/**
	 * clears (resets) form value. 
	 * 
	 * Usefull for clearing submited form value
	 */
	function clear(){
		$this->valid = null;
		$this->validValue = null;
	}
	/**
	 * add validator
	 * 
	 * @param Kod_Validator $validator new validator
	 * 
	 * If $validator is of type Required_Validator it will
	 * replace current Required_Validator. SO it is never possible
	 * to have two required validators.
	 */
	function addValidator(Kod_Validator $validator){
		if($validator instanceof Required_Validator)
			$this->required = $validator;
		else
			array_push($this->validators, $validator);
	}
	/**
	 * Add filter
	 * 
	 * Filter difers from validator because it filters value. For example
	 * trim filter removes white space from begining and from the end of string.
	 * After value is filtered it is passed to validators for validation in validate
	 * member function.
	 */
	function addFilter(Kod_Filter $filter){
		array_push($this->filters, $filter);
	}
	/**
	 * returns validation errors as array
	 * 
	 * @return array 
	 */
	function getErrors(){
		return $this->errors;
	}
	/**
	 * returns Id of element
	 * 
	 * @return string
	 */
	function getId(){
		return $this->id;
	}
	/**
	 * this function process passed array as html attributes.
	 * 
	 * @param array $attributes array where key is attribute name and
	 * value is attribute value
	 */
	protected function processAttributes($attributes){
		$cs = "";
		if(is_array($attributes))
			foreach($attributes as $name => $value)
				$cs .= " " . $name . "=\"" . $value . "\"";
		return $cs;
	}
	/**
	 * function which is responsible for rendering (displaying) form
	 * element
	 * 
	 * @param Kod_Attributes this element holds attributes ususlay style and class
	 * for diferent element status(error, required, normal, ....)
	 */
	abstract function display(Kod_Attributes $attributes = null);
	/**
	 * set this element as required
	 * 
	 * @param string $message mesage which will be displayed for validation 
	 * error
	 */
	function setRequired($message){
		$this->required = new Required_Validator($message);
	}
	/**
	 * check if element is required
	 */
	function isRequired(){
		if($this->required != null)
			return true;
		return false;
	}
	/**
	 * returns error message which should be used if value for this field
	 * is required
	 * 
	 * @return mixed if it is required than string with message oterwise null
	 */
	function getRequiredError(){
		if($this->required != null)
			return $this->required->getErrors();
		return null;
	}
	/**
	 * check is this field is valid
	 * 
	 * should be used after validate method to get proper results. If not
	 * ti will be always true
	 * 
	 * @return boolean
	 */
	function isValid(){
		$valid = true;
		if(count($this->errors) > 0)
			$valid = false;
		return $valid;
	}
	/**
	 * returns value of this field
	 * 
	 * if form is submited and it is valid you get valid value
	 * if is not valid you get not valid value
	 * if form is not submite you get default value
	 */
	function getValue(){
		$value = $this->defaultValue;
		if($this->form->isSubmited()){
			if($this->isValid())
				$value = $this->validValue;
			else
				$value = $this->form->getRaw($this->id);
		}
		return $value;
	}
	/**
	 * sets display name
	 * 
	 * @param string $name new display name
	 */
	function setDisplayName($name){
		$this->displayName = $name;
	}
	/**
	 * gets display name
	 * 
	 * @return string
	 */
	function getDisplayName(){
		if($this->displayName != null)
			return $this->displayName;
		return $this->id;
	}
	/**
	 * sets default value
	 * 
	 * @param string $value new default value
	 */
	function setDefaultValue($value){
		$this->defaultValue = $value;
	}
}
/**
 * implements submit button
 * 
 * this is submit button form element
 */
class Kod_Submit extends Kod_Form_Element{
	function __construct($id, $label){
		parent::__construct($id);
		$this->displayName = $label;
	}
	/**
	 * submit is always valid so here is overloaded Kod_Form_Element method
	 * 
	 * @return boolean always true
	 */
	function validate(){
		return true;
	}
	/**
	 * default display (rendering)
	 */
	function display(Kod_Attributes $attributes = null){
		if($attributes === null)
			$attributes = new Kod_Attributes();
		?>
<input name="<?php print $this->id?>" type="submit"<?php print $this->processAttributes($attributes->normal);?> value="<?php print $this->displayName;?>" />
<?php
	}
	/**
	 * check if this button was clicked (submited)
	 * 
	 * ususally used by parent form
	 */
	function isSubmited(){
		$method = $this->form->getMethod();
		if($method == 0)
			if(isset($_POST[$this->id]))
				return true;
		if($method == 1)
			if(isset($_GET[$this->id]))
				return true;
		return false;
	}
}

/**
 * this is implemetation of text (password) field (Kod_Form_Element) 
 */
class Kod_Field extends Kod_Form_Element{
	/**
	 * @var string holds type (text or password or hidden)
	 */
	protected $type;
	/**
	 * 
	 * var integersize attribute used when displaying this element
	 */
	protected $displaySize;
	/**
	 * @var integer maxsize atribute used when displaying this element
	 */
	protected $maxSize;
	
	const text = "text";
	const password = "password";
	const hidden = "hidden";
	/**
	 * create field object
	 * 
	 * @param string $id id of element @see Kod_Form_Element::id
	 * @param string $displayName field name which will be displayed
	 * @param string $type type of this element (text, password, hidden). See
	 * constants of this class
	 * @param string $defaultValue value which will be displayed to user in this
	 * element. If form is submited submited value will be displayed.
	 * @param string $required if this is set than the element will be required and the suplied text will
	 * be error message for validation (if field do not contain value)
	 */
	function __construct($id, $displayName, $type = Kod_Field::text, $defaultValue = null, $required = null){
		parent::__construct($id, $defaultValue, $required);
		$this->type = $type;
		$this->displaySize = 20;
		$this->maxSize = 0;
		$this->displayName = $displayName;
	}
	/**
	 * default display function
	 */
	function display(Kod_Attributes $attributes = null){
		$value = $this->getValue();
		$str = "<input name=\"" . $this->id . "\" type=\"" . $this->type . "\" value=\"" . $value . "\""; 
		
		if($this->type != Kod_Field::hidden){
			if($attributes == null)
				$attributes = new Kod_Attributes();
			$attr = $attributes->normal;
			if($this->isRequired())
				$attr = $attributes->required;
			if(!$this->isValid())
				$attr = $attributes->error;
			$str .= $this->processAttributes($attr);
			
			$str .= " size=\"" . $this->displaySize . "\"";
			
			if($this->type == Kod_Field::text && $this->maxSize > 0)
				$str .= " maxsize=\"" . $this->maxSize . "\"";
		}
		
		$str .= " />";
		return $str;
	}
	/**
	 * set size
	 * 
	 * this sets attribute size of this field (html)
	 */
	function setDisplaySize($size){
		$this->displaySize = $size;
	}
	/**
	 * set maxsize 
	 * 
	 * this sets maxsize attribute of this field (html)
	 */
	function setMaxSize($maxsize){
		$this->maxSize = $maxsize;
	}
}
/**
 * textarea implementation
 */
class Kod_Text extends Kod_Form_Element{
	protected $cols;
	protected $rows;
	/**
	 * creates textarea object
	 * 
	 * @param string $id element id
	 * @param string $displayName display name which will be associed 
	 * with textarea
	 * @param integer $rows defines size of text area. Same as rows in textarea html tag
	 * @param integer $cols defines size of text area. Same as cols in textarea html tag
	 * @param string $defaultValue sets default value 
	 */
	function __construct($id, $displayName, $rows = 0, $cols = 0, $defaultValue = null ){
		parent::__construct($id, $defaultValue);
		self::setDisplayName($displayName);
		$this->cols = $cols;
		$this->rows = $rows;
	}
	/**
	 * display function
	 * 
	 * produces html reprezentation
	 */
	function display(Kod_Attributes $attributes = null){
		if($attributes == null)
			$attributes = new Kod_Attributes();
		$attr = $attributes->normal;
		if($this->isRequired())
			$attr = $attributes->required;
		if(!$this->isValid())
			$attr = $attributes->error;
		$str = "<textarea name=\"" . $this->id . "\"";
		$str .= $this->processAttributes($attr);
		if($this->cols >0)
			$str .= " cols=\"" . $this->cols . "\"";
		if($this->rows >0)
			$str .= " rows=\"" . $this->rows . "\"";
		$str .= ">" . $this->getValue() . "</textarea>";
		return $str;
	}
	/**
	 * sets how many cols this textarea should have
	 */
	function setCols($cols){
		$this->cols = $cols;
	}
	/**
	 * sets how many rows thia textarea should have
	 */
	function setRows($rows){
		$this->rows = $rows;
	}
}

/**
 * select element implementation
 */
class Kod_Select extends Kod_Form_Element{
	private $options;
	private $error;
	/**
	 * Creates select element
	 * 
	 *  @param string $id element idenftifer
	 *  @param string $displayName name which is used when displaying to user
	 *  @param $error this is error message if element value is not recognised.
	 *  When there is no oprion with this value. 
	 */
	function __construct($id, $displayName, $error){
		parent::__construct($id);
		self::setDisplayName($displayName);
		$this->options = array();
		$this->error = $error;
	}
	/**
	 * Overloaded function because this element does not use validators and filters.
	 * 
	 * It only checks if there is option with value same as POSTed value
	 */
	function validate(){
		$valid = true;
		//check if form is submited
		if($this->form != null && $this->form->isSubmited()){
			$this->rawValue = $this->form->getRaw($this->id);//get raw submited data
			if(!in_array($this->rawValue, $this->options)){
				$valid = false;
				array_push($this->errors, $this->error);
			}
			if($valid)
				$this->validValue = $this->rawValue;
		}
		return $valid;
	}
	/**
	 * Adds new option
	 * 
	 * @param string $displayName name which will be wissivle to user
	 * @param string $value value of option
	 * @param boolena $default set this to true if this is defult element. 
	 * Default option will be displayed. If you add more then one default option 
	 * then last added will be default.
	 */
	function addOption($displayName, $value, $default = false){
		$this->options[$displayName] = $value;
		if($default == true || count($this->options) == 1 )
			$this->defaultValue = $value;
	}
	/**
	 * Basic rendering of these element
	 */
	function display(Kod_Attributes $attributes = null){
		if($attributes == null)
			$attributes = new Kod_Attributes();
		$str = "<select name=\"" . $this->id . "\"";
		if($attributes == null)
				$attributes = new Kod_Attributes();
			$attr = $attributes->normal;
			if(!$this->isValid())
				$attr = $attributes->error;
			$str .= $this->processAttributes($attr);
		$str .= ">";
		foreach($this->options as $optName => $optVal){
			$str .= "<option value=\"" . $optVal . "\"";
			if($this->getValue() == $optVal)
				$str .= "selected=\"selected\"";
			$str .= ">" . $optName . "</option>";
		}
		$str .="</select>";
		return $str; 
	}
}

/**
 * base Validator class
 * 
 * every validator should extend this class
 */
abstract class Kod_Validator{
	private $error;
	/**
	 * constructor which takes error message as paramatar. This error message
	 * will be used if value is not valid. 
	 */
	public function __construct($error){
		$this->error = $error;
	}
	/**
	 * abstract method which do validation
	 * 
	 * this methos should be overloaded to implement validation
	 */
	abstract public function validate($value);
	/**
	 * returns validation error
	 * 
	 * this function will return error eaven if value is valid.
	 */
	public function getError(){
		return $this->error;
	}
}
/**
 * required filed validator
 * 
 * this is little special validator and it is used by form elements in
 * special way. Form elements can have only one required validator.
 */
class Required_Validator extends Kod_Validator{
	public function __construct($message){
		parent::__construct($message);
	}
	function validate($value){
		if($value == "" || $value == null)
			return false;
		return true;
	}
}
/**
 * this is regex validator
 * 
 * this is base validator for many validator
 * but it is most powefull one
 */
class Regex_Validator extends Kod_Validator{
	private $pattern;
	/**
	 * creates regex validator
	 * 
	 * @param string $patter rexeg patter to match
	 * @param string $error validation error
	 */
	public function __construct($pattern, $error){
		parent::__construct($error);
		$this->pattern = $pattern;
	}
	/**
	 * validation
	 * 
	 * it is required to note here that all value should match pattern
	 */
	public function validate($value){
		$matched = null;
		preg_match($this->pattern, $value, $matched);
		if($matched[0] == $value)
			return true;
		return false;
	}
}

/**check how many chars has string*/
class Size_Validator extends Kod_Validator{
	protected $maxSize;
	protected $minSize;
	public function __construct($maxSize, $minSize, $error){
		parent::__construct($error);
		$this->maxSize = $maxSize;
		$this->minSize = $minSize;
	}
	public function validate($value){
		$value = strlen($value);
		if($this->maxSize == 0){
			if($value >= $this->minSize)
				return true;
		}
		else if($this->minSize == 0){
			if($value <= $this->maxSize)
				return true;
		}
		else{
			if($value <= $this->maxSize && $value >= $this->minSize)
				return true;
		}
		return false;
	}
}

class MaxMin_Validator extends Kod_Validator{
	protected $max;
	protected $min;
	public function __construct($max, $min, $error){
		parent::__construct($error);
		$this->max = $max;
		$this->min = $min;
	}
	public function validate($value){
		if($value < $this->max && $value > $this->min)
			return true;
		return false;
	}
}
/**
 * this is base class for filters
 * 
 * filters are elements which filter value before validation
 */
abstract class Kod_Filter{
	/**
	 * this method should be impleneted to create filter
	 */
	abstract public function filter($string);
}
/**
 * one of most popular filters which will remove spaces at beginig
 * and at the end of string
 */
class Trim_Filter extends Kod_Filter{
	public function filter($string){
		return trim($string);
	}
}

/**
 * this class implements attributes for different states of element
 * 
 * it is used in display function of all elements.
 */
class Kod_Attributes{
	protected $attributes;
	const required = "required";
	const normal = "normal";
	const error = "error";
	function __construct(){
		$this->attributes = array();
	}
	function __set($type, $value){
		if($type == Kod_Attributes::required || $type == Kod_Attributes::normal || $type == Kod_Attributes::error){
			if(is_array($value))
				$this->attributes[$type] = $value;
		}
	}
	function __get($type){
		if($type == Kod_Attributes::required || $type == Kod_Attributes::normal || $type == Kod_Attributes::error){
			if(isset($this->attributes[$type]))
				return $this->attributes[$type];
		}
		return null;
	}
	function addAttributes($attributes, $type = null){
		if($type == Kod_Attributes::required || $type === null)
			$this->attributes[Kod_Attributes::required] = array_merge($this->attributes[Kod_Attributes::required], $attributes);
		else if($type == Kod_Attributes::normal || $type === null)
			$this->attributes[Kod_Attributes::normal] = array_merge($this->attributes[Kod_Attributes::normal], $attributes);
		else if($type == Kod_Attributes::error || $type === null)
			$this->attributes[Kod_Attributes::error] = array_merge($this->attributes[Kod_Attributes::error], $attributes);
	}
}
?>
